﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Windows.Forms;
using CUETools.Codecs;
using CUETools.Codecs.FLAKE;

namespace GoogleSpeech
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            txtPath.Text = Settings.Instance.wavName;
        }
        
        private delegate void DoWorkDelegate(String filePath);
              
        private void SendFileRun(String filePath)
        {
          // Disable the button
          DisableStartButtons();
          // Create delegate and make async call
          DoWorkDelegate worker = new DoWorkDelegate(SendFile);
          worker.BeginInvoke(filePath, new AsyncCallback(DoWorkComplete), worker);
        }

        private void EnableStartButtons()
        {
          if (InvokeRequired)
          {
            Invoke(new MethodInvoker(EnableStartButtons));
          }
          else
          {
            btnBrowse.Enabled = true;
            btnSearch.Enabled = true;
          }
          
        }

        private void DisableStartButtons()
        {
          btnBrowse.Enabled = false;
          btnSearch.Enabled = false;
        }


        private void DoWorkComplete(IAsyncResult workID)
        {
          EnableStartButtons();
          DoWorkDelegate worker = workID.AsyncState as DoWorkDelegate;
          worker.EndInvoke(workID);
          //ReportOnProgress(100, "Запрос произведен");
        }


        private delegate void ReportOnProgressDelegate(int progress, string msg);

        private void ReportOnProgress(int progress, string msg)
        {
          if (InvokeRequired)
          {
            Invoke(new ReportOnProgressDelegate(ReportOnProgress),
              new object[] { progress, msg });
            return;
            ;
          }
                    
            this.Text = Settings.Instance.AppTitle + " —  " + msg;                      
        }
       
        private void SendFile(String filePath)
        {          
          try
          {
            ReportOnProgress(10, "Идет запрос");
            String responseFromServer = GoogleVoice.GoogleSpeechRequest(filePath, Settings.Instance.tmpName);

            
            AddLog(responseFromServer + System.Environment.NewLine);
            JSon.RecognitionResult result = JSon.Parse(responseFromServer);
            if (result.hypotheses.Length > 0)
            {
              JSon.RecognizedItem item = result.hypotheses.First();
              //listBox1.Items.Add(String.Format("{0:0.000} ; {1}", item.confidence, item.utterance));
              AddToList(String.Format("{0:0.000} ; {1}", item.confidence, item.utterance));
            }
            else
            {
              AddToList(String.Format("None"));
              //listBox1.Items.Add(String.Format("None"));
            }

            ReportOnProgress(100, "Запрос успешно выполнен");
          }
          catch (Exception e)
          {
            ReportOnProgress(100, "Ошибка запроса: " + e.Message);
            AddLog(e.ToString());            
          }
                     
        }

        private delegate void AddDelegate(String log);
        private void AddLog(string log)
        {         
            if (InvokeRequired)
            {
              Invoke(new AddDelegate(AddLog), new object[] { log });
              return; 
            }
         
          txtLog.Text += log;
        }

      private void AddToList(String item)
      {
        if (InvokeRequired)
        {
          Invoke(new AddDelegate(AddToList), new object[] { item });
          return; 
        }
        listBox1.Items.Add(item);
      }


        private void button1_Click(object sender, EventArgs e)
        {
            SendFileRun(Settings.Instance.wavName);
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = "Open Wave File";
            openFileDialog1.Filter = "Wave files (*.wav)|*.wav";
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
            {
                return;
            }

            string sFilePath;
            sFilePath = openFileDialog1.FileName;
            if (sFilePath == "")
            {
                return;
            }
          
            Settings.Instance.wavName = sFilePath;
            txtPath.Text = sFilePath;
            SendFileRun(Settings.Instance.wavName);
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            txtLog.SelectionStart = txtLog.Text.Length;
            txtLog.ScrollToCaret();
            txtLog.Refresh();
        }

        private void txtPath_TextChanged(object sender, EventArgs e)
        {
            Settings.Instance.wavName = txtPath.Text;
        }
    }
}
